package plugin.simple.evaluation;

import java.awt.Component;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;

import edu.cmu.side.simple.ModelEvaluationPlugin;
import edu.cmu.side.simple.SimpleTrainingResult;
import edu.cmu.side.simple.feature.Feature;
import edu.cmu.side.simple.feature.FeatureTable;

public class HorizontalComparison extends ModelEvaluationPlugin{

	static Map<String, Map<Feature, Double>> cache = new HashMap<String, Map<Feature, Double>>();
	@Override
	public Map<Feature, Double> evaluateModelFeatures(
			SimpleTrainingResult model, Map<String, String> settings) {
		String pred = settings.get("pred");
		String act = settings.get("act");
		String key = model.toString()+"_"+pred+"_"+act;
		try{
			if(!cache.containsKey(key)){
				Map<Feature, Double> eval = new HashMap<Feature, Double>(model.getFeatureTable().getFeatureSet().size());
				List<Integer> hitDocs = model.getConfusionMatrixCell(pred, act);
				List<Integer> actDocs = model.getConfusionMatrixCell(act, act);
				FeatureTable ft = model.getEvaluationTable();
				Set<Feature> feats = ft.getFeatureSet();
				for(Feature f : feats){
					Double val = model.getAverageValue(hitDocs, f) - model.getAverageValue(actDocs, f);
					eval.put(f, val);
				}				
				cache.put(key, eval);
			}
		}catch(Exception e){	
			e.printStackTrace();
		}
		return cache.get(key);
	}

	@Override
	public String getOutputName() {
		return "horiz";
	}

	@Override
	public String toString(){
		return "Horizontal Comparison";
	}

	@Override
	protected Component getConfigurationUIForSubclass() {
		// TODO Auto-generated method stub
		return null;
	}

}
