package plugin.simple.features;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.w3c.dom.Element;

import se.datadosen.component.RiverLayout;

import edu.cmu.side.dataitem.DocumentListInterface;
import edu.cmu.side.simple.FeaturePlugin;
import edu.cmu.side.simple.feature.Feature;
import edu.cmu.side.simple.feature.FeatureHit;
import edu.cmu.taghelpertools.middle_layer.FeatureSelector;
import edu.cmu.taghelpertools.middle_layer.OptionsModel;
import edu.cmu.taghelpertools.tokenizer.Tokenizer;
import edu.cmu.taghelpertools.util.HelperFunctions;
import gnu.trove.THashMap;

public class NegotiationExtractor extends FeaturePlugin {

	/**Boolean features for every TagHelper setting combination. */
	static Boolean isUni = true; static Boolean isBi = false;
	static Boolean optBool = true; static Boolean isLineLen = false;
	
	/**Swing components for every TagHelper setting combination. */
	JCheckBox uniBox; JCheckBox biBox; JCheckBox remAdjBox;
	JCheckBox boolBox; JCheckBox lineLenBox;
	
	JPanel parentPanel;
	JPanel featureTypesPanel;
	JPanel featureOptionsPanel;
	
	OptionsModel o;
	
	@Override
	public Collection<FeatureHit> extractFeatureHitsForSubclass(
			DocumentListInterface documents, JLabel progress) {
		Collection<FeatureHit> hits = new HashSet<FeatureHit>();
		ArrayList<String> annot = documents.getAnnotationArray();
		for(String name : documents.allAnnotations().keySet()){
			try{
				ArrayList<String> features = documents.allAnnotations().get(name);
				ArrayList<Double> doubles = new ArrayList<Double>();
				if(features != annot){
					Feature.Type type = Feature.Type.NUMERIC;
					boolean alwaysBoolean = true;
					for(int i = 0; i < features.size(); i++){
						if(!(features.get(i).trim().equalsIgnoreCase("true")||features.get(i).trim().equalsIgnoreCase("false"))){
							alwaysBoolean = false;
						}
						try{
							Double d = Double.parseDouble(features.get(i).trim());
							doubles.add(d);
						}catch(Exception e){
							type = Feature.Type.NOMINAL;
						}
					}
					if(alwaysBoolean){
						type = Feature.Type.BOOLEAN;
					}
					Feature f;
					if(type == Feature.Type.NOMINAL){
						Set<String> values = new HashSet<String>();
						for(int i = 0; i < features.size(); i++){
							values.add(features.get(i));
						}
						f = Feature.fetchFeature(getOutputName(), name, values);
					}else{
						f = Feature.fetchFeature(getOutputName(), name, type);
					}
					for(int i = 0; i < features.size(); i++){
						switch(type){
						case NOMINAL:
							System.out.println("Adding nominal " + name);
							hits.add(new FeatureHit(f,features.get(i),i));
							break;
						case NUMERIC:
							System.out.println("Adding numeric " + name);
							hits.add(new FeatureHit(f,doubles.get(i),i));
							break;
						case BOOLEAN:
							if(features.get(i).trim().equalsIgnoreCase("true")){
								System.out.println("Adding boolean " + name);
								hits.add(new FeatureHit(f,Boolean.TRUE,i));
							}
							break;
						}
					}
				}
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		try{
			FeatureSelector fs = new FeatureSelector(o);
			List<String> corpus = documents.getCoveredTextList();
			double time1 = System.currentTimeMillis();
			Map<String, Feature> allFeatures = new TreeMap<String, Feature>();
			int index = 0;
			for(String document : corpus){
				String doc = HelperFunctions.replacePuncWithSpecialTokens(document.toString(), Tokenizer.LANGS[Tokenizer.LANG_ENGLISH]);
				THashMap feats = fs.getFeatureSet(doc);
				for(Object feat : feats.keySet()){
					if(remAdjBox.isSelected() && feat.toString().contains("_")){
						String[] big = feat.toString().split("_");
						if(big[0].charAt(0) == big[1].charAt(0)) continue;
					}
					Feature f;
					if(!allFeatures.containsKey(feat.toString())){
						f = Feature.fetchFeature(getOutputName(), feat.toString(), optBool?Feature.Type.BOOLEAN:Feature.Type.NUMERIC);
						allFeatures.put(feat.toString(), f);	
					}else{
						f = allFeatures.get(feat.toString());
					}
					Double value = (Double)feats.get(feat);
					if(optBool && !feat.toString().equals(FeatureSelector.LINE_LENGTH)){
						hits.add(new FeatureHit(f, value > 0.0 ? Boolean.TRUE : Boolean.FALSE, index));
					}else{
						hits.add(new FeatureHit(f, value, index));				
					}
				}
				index++;
			}
			double time2 = System.currentTimeMillis();
			System.out.println("Total time creating map: " + (time2-time1));			
		}catch(Exception e){
			e.printStackTrace();
		}
		System.out.println(hits.size() + " hits.");
		return hits;
	}

	@Override
	public String toString(){
		return "Negotiation Country Extractor";
	}

	@Override
	public String getOutputName() {
		return "ng";
	}
		
	public NegotiationExtractor(){
		init();
	}
	
	/**
	 * Set binary unigrams as the default.
	 */
	public void init(){
		uniBox = new JCheckBox("unigrams", true);
		biBox = new JCheckBox("bigrams");
		boolBox = new JCheckBox("treat above features as binary?", true);
		lineLenBox = new JCheckBox("line length");
		remAdjBox = new JCheckBox("remove adjacent same speaker");
		uniBox.addActionListener(        new ActionListener(){ public void actionPerformed(ActionEvent e){ isUni = uniBox.isSelected();}});
		biBox.addActionListener(         new ActionListener(){ public void actionPerformed(ActionEvent e){ isBi = biBox.isSelected();}});
		boolBox.addActionListener(       new ActionListener(){ public void actionPerformed(ActionEvent e){ optBool = boolBox.isSelected();}});
		lineLenBox.addActionListener(    new ActionListener(){ public void actionPerformed(ActionEvent e){ isLineLen = lineLenBox.isSelected();}});
		
		parentPanel = new JPanel();
		parentPanel.setLayout(new BoxLayout(parentPanel, BoxLayout.X_AXIS));
		featureTypesPanel = new JPanel(new RiverLayout());
		featureOptionsPanel = new JPanel(new RiverLayout());
		
		
		featureTypesPanel.add(uniBox, "br hfill");
		featureTypesPanel.add(biBox, "br hfill");
		featureTypesPanel.add(lineLenBox, "br hfill");
		featureOptionsPanel.add(boolBox, "br hfill");
		featureOptionsPanel.add(remAdjBox, "br hfill");
		
		parentPanel.add(featureTypesPanel);
		parentPanel.add(featureOptionsPanel);
	}

	@Override
	protected Component getConfigurationUIForSubclass() {
		return parentPanel;
	}

	@Override
	public void uiToMemory(){
		System.out.println("UI copying to memory." + isUni + ", " + isBi);
		o = new OptionsModel();
		o.setPunctuation(false);
		o.setUnigrams(isUni);
		o.setBigrams(isBi);
		o.setPosBigrams(false);
		o.setLineLength(isLineLen);
		o.setContainsNonStopword(false);
		o.setRemoveStopwords(false);
		o.setStem(false);
		o.setStopwordsAll(true);
	}
	public void configureFromFile(String filename){
		
	}

}
