package plugin.simple.features;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.w3c.dom.Element;

import com.yerihyo.yeritools.io.FileToolkit;

import se.datadosen.component.RiverLayout;

import edu.cmu.side.SimpleWorkbench;
import edu.cmu.side.dataitem.DocumentListInterface;
import edu.cmu.side.simple.FeaturePlugin;
import edu.cmu.side.simple.feature.Feature;
import edu.cmu.side.simple.feature.FeatureHit;
import edu.cmu.side.simple.feature.Feature.Type;
import edu.cmu.side.simple.feature.LocalFeatureHit;
import edu.cmu.side.simple.newui.features.FeaturePluginPanel;
import edu.cmu.side.simple.newui.machinelearning.LearningConfigPanel;

public class StretchPatterns extends FeaturePlugin
{
	JPanel config = new JPanel();
	JTextField gapMin = new JTextField(3);
	JTextField gapMax = new JTextField(3);
	JTextField patMin = new JTextField(3);
	JTextField patMax = new JTextField(3);

	JTextArea categoryNames = new JTextArea(5,50);
	Integer gMin = 0;
	Integer gMax = 3;
	Integer pMin = 1;
	Integer pMax = 5;

	boolean halt = false;
	
	static Map<String, Set<String>> categories = new TreeMap<String, Set<String>>();
	
	private static class PatternInfo implements Comparable<PatternInfo>
	{
		public String pattern;
		public Collection<int[]> hitRanges = new ArrayList<int[]>();
		
		PatternInfo(String p, int start, int end)
		{
			this.pattern = p;
			hitRanges.add(new int[]{start, end});
		}

		@Override
		public int compareTo(PatternInfo o)
		{
			return pattern.compareTo(o.pattern);
		}
	}
	
	@Override
	public String toString(){
		return "Stretch Patterns";
	}

	public StretchPatterns(){
		config.setLayout(new RiverLayout());
		config.add("center", new JLabel("Min Gap Length:"));
		config.add("center", gapMin);
		config.add("center", new JLabel("Max Gap Length:"));
		config.add("center", gapMax);
		gapMin.setText("0");
		gapMax.setText("3");
		config.add("br center", new JLabel("Min Pattern Length:"));
		config.add("center", patMin);
		config.add("center", new JLabel("Max Pattern Length:"));
		config.add("center", patMax);
		patMin.setText("1");
		patMax.setText("5");
		config.add("br center", new JLabel("Categories:"));
		JButton loadCatsButton = new JButton("Load");
		loadCatsButton.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser();
				int selection = chooser.showOpenDialog(config);
				if(selection!=JFileChooser.APPROVE_OPTION){
					return;
				}
				getCategories(chooser.getSelectedFile());
			}
		});
		config.add("center", loadCatsButton);
		config.add("br center", new JScrollPane(categoryNames));
		categoryNames.setEditable(false);
	}
	
	public void getCategories(File cats){
		try{
			BufferedReader read = new BufferedReader(new FileReader(cats));			
			String line;
			boolean newCat = true;
			String cat = "";
			while((line = read.readLine()) != null){
				if(line.length() == 0){
					newCat = true;
				}else if(newCat){
					cat = line;
					newCat = false;
				}else{
					if(!categories.containsKey(line)){
						categories.put(line, new TreeSet<String>());
					}
					categories.get(line).add(cat);
				}
			}
			read.close();
			StringBuilder sb = new StringBuilder();
			for(String c : categories.keySet()){
				sb.append(c + " {");
				for(String l : categories.get(c)){
					sb.append(l + ",");
				}
				sb.deleteCharAt(sb.length()-1);
				sb.append("}\n");
			}
			categoryNames.setText(sb.toString());
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	@Override
	public void uiToMemory(){
		gMin = Integer.parseInt(gapMin.getText());
		gMax = Integer.parseInt(gapMax.getText());
		pMin = Integer.parseInt(patMin.getText());
		pMax = Integer.parseInt(patMax.getText());
	}

	@Override
	public Collection<FeatureHit> extractFeatureHitsForSubclass(
			DocumentListInterface documents, JLabel progress) {
		Collection<FeatureHit> docHits = new HashSet<FeatureHit>();
		List<String> docs = documents.getCoveredTextList();
		Map<String, Feature> allFeatures = new TreeMap<String, Feature>();
		int index = 0;
		for(String doc : docs){
			if(halt){
				halt = false;
				return null;
			}
			progress.setText("Extracting from document " + index);
			String[] words = doc.split("\\s+");
			Set<PatternInfo> hits = extract(words,gMin,gMax,pMin,pMax);
			for(PatternInfo hit : hits){
				if(!allFeatures.containsKey(hit.pattern)){
					allFeatures.put(hit.pattern, Feature.fetchFeature(getOutputName(), hit.pattern, Feature.Type.BOOLEAN));
				}
//				docHits.add(new FeatureHit(allFeatures.get(hit), Boolean.TRUE, index));
				docHits.add(new LocalFeatureHit(allFeatures.get(hit.pattern), Boolean.TRUE, index, hit.hitRanges));
			}
			index++;
		}
		progress.setText("");
		
		
		return docHits;
	}

	

	public static void main(String[] args){
		System.out.println(extract(new String[]{"gK2","fK1","goo","fK2","gK1","gA2","fA1","goo","foo","goo"}, 0,3,1,5).size());
	}

	public static void extract(Set<PatternInfo> patterns, boolean justGap, String present, int minGapSize, int maxGapSize, int minPatternSize, int currentPatternSize, int remainingPatternSize, String[] words, int currentIndex, int startingIndex){
		if(currentPatternSize >= minPatternSize && present.length() > 0 && !present.endsWith("[GAP] ") && !present.startsWith("[GAP] ")){
			patterns.add(new PatternInfo(present, startingIndex, currentIndex+1));
		}
		if(remainingPatternSize > 0 && currentIndex < words.length){
			extract(patterns, false, present+words[currentIndex]+" ", minGapSize, maxGapSize, minPatternSize, currentPatternSize+1, remainingPatternSize-1, words, currentIndex+1, startingIndex);
			if(categories.containsKey(words[currentIndex])){
				for(String c : categories.get(words[currentIndex])){
					extract(patterns, false, present+c+" ", minGapSize, maxGapSize, minPatternSize, currentPatternSize+1, remainingPatternSize-1, words, currentIndex+1, startingIndex);					
				}
			}
			for(int i = minGapSize; i <= maxGapSize; i++){
				if(currentIndex+i < words.length && !justGap){
					extract(patterns, true, present+"[GAP] ",minGapSize,maxGapSize,minPatternSize, currentPatternSize+1, remainingPatternSize-1,words,currentIndex+i, startingIndex);
				}
			}
		}
	}

	public static Set<PatternInfo> extract(String[] words, int gMin, int gMax, int pMin, int pMax){
		Set<PatternInfo> patterns = new TreeSet<PatternInfo>();
		for(int i = 0; i < words.length; i++){
			extract(patterns,false,"",gMin,gMax,pMin,0,pMax,words,i, i);
		}
		return patterns;
	}

	@Override
	public String getOutputName() {
		return "str";
	}

	@Override
	protected Component getConfigurationUIForSubclass() {
		return config;
	}

	public void configureFromFile(String filename){
		try{
			BufferedReader in = new BufferedReader(new FileReader(filename));		
			gapMin.setText(in.readLine());
			gapMax.setText(in.readLine());
			patMin.setText(in.readLine());
			patMax.setText(in.readLine());
			in.close();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void stopWhenPossible(){
		halt = true;
	}
}
