package plugin.simple.features;

import java.awt.Component;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import org.w3c.dom.Element;

import com.yerihyo.yeritools.xml.XMLToolkit;

import se.datadosen.component.RiverLayout;

import edu.cmu.side.dataitem.DocumentListInterface;
import edu.cmu.side.simple.FeaturePlugin;
import edu.cmu.side.simple.feature.Feature;
import edu.cmu.side.simple.feature.FeatureHit;
import edu.cmu.side.simple.feature.Feature.Type;
import edu.cmu.side.simple.feature.LocalFeatureHit;
import edu.cmu.taghelpertools.middle_layer.FeatureSelector;
import edu.cmu.taghelpertools.middle_layer.OptionsModel;
import edu.cmu.taghelpertools.tokenizer.IndexedFeature;
import edu.cmu.taghelpertools.tokenizer.Tokenizer;
import edu.cmu.taghelpertools.util.HelperFunctions;
import gnu.trove.THashMap;

public class TagHelperTools extends FeaturePlugin{

	/**Boolean features for every TagHelper setting combination. */
	static Boolean isPunct = false; static Boolean isUni = true; static Boolean isBi = false; static Boolean isTri = false;
	static Boolean isPosBi = false; static Boolean optBool = true; static Boolean isLineLen = false;
	static Boolean isContNonStop = false; static Boolean optRemoveStop = false; static Boolean optStem = false;

	/**Swing components for every TagHelper setting combination. */
	transient JCheckBox punctBox; transient JCheckBox uniBox; transient JCheckBox biBox; transient JCheckBox triBox;
	transient JCheckBox posBiBox; transient JCheckBox boolBox; transient JCheckBox lineLenBox;
	transient JCheckBox contNonStopBox; transient JCheckBox removeStopBox; transient JCheckBox stemBox;

	transient JPanel parentPanel;
	transient JPanel featureTypesPanel;
	transient JPanel featureOptionsPanel;

	public static boolean halt = false;
	OptionsModel o;

	public TagHelperTools(){
		init();
	}

	/**
	 * Set binary unigrams as the default.
	 */
	public void init(){
		punctBox = new JCheckBox("Punctuation");
		uniBox = new JCheckBox("Unigrams", true);
		biBox = new JCheckBox("Bigrams");
		triBox = new JCheckBox("Trigrams");
		posBiBox = new JCheckBox("POS Bigrams");
		boolBox = new JCheckBox("Treat above features as binary?", true);
		lineLenBox = new JCheckBox("Line length");
		contNonStopBox = new JCheckBox("Contains NON-stopwords");
		removeStopBox = new JCheckBox("Remove Stopwords");
		stemBox = new  JCheckBox("Stem");

		punctBox.addActionListener(      new ActionListener(){ public void actionPerformed(ActionEvent e){ isPunct = punctBox.isSelected();}});
		uniBox.addActionListener(        new ActionListener(){ public void actionPerformed(ActionEvent e){ isUni = uniBox.isSelected();}});
		biBox.addActionListener(         new ActionListener(){ public void actionPerformed(ActionEvent e){ isBi = biBox.isSelected();}});
		triBox.addActionListener(         new ActionListener(){ public void actionPerformed(ActionEvent e){ isTri = triBox.isSelected();}});
		posBiBox.addActionListener(      new ActionListener(){ public void actionPerformed(ActionEvent e){ isPosBi = posBiBox.isSelected();}});
		boolBox.addActionListener(       new ActionListener(){ public void actionPerformed(ActionEvent e){ optBool = boolBox.isSelected();}});
		lineLenBox.addActionListener(    new ActionListener(){ public void actionPerformed(ActionEvent e){ isLineLen = lineLenBox.isSelected();}});
		contNonStopBox.addActionListener(new ActionListener(){ public void actionPerformed(ActionEvent e){ isContNonStop = contNonStopBox.isSelected();}});
		removeStopBox.addActionListener( new ActionListener(){ public void actionPerformed(ActionEvent e){ optRemoveStop = removeStopBox.isSelected();}});
		stemBox.addActionListener(       new ActionListener(){ public void actionPerformed(ActionEvent e){ optStem = stemBox.isSelected();}});

		parentPanel = new JPanel();
		parentPanel.setLayout(new BoxLayout(parentPanel, BoxLayout.X_AXIS));
		featureTypesPanel = new JPanel(new RiverLayout());
		featureOptionsPanel = new JPanel(new RiverLayout());


		featureTypesPanel.add(uniBox, "br hfill");
		featureTypesPanel.add(biBox, "br hfill");
		featureTypesPanel.add(triBox, "br hfill");
		featureTypesPanel.add(posBiBox, "br hfill");
		featureTypesPanel.add(punctBox, "br hfill");
		featureTypesPanel.add(lineLenBox, "br hfill");
		featureTypesPanel.add(contNonStopBox, "br hfill");

		featureOptionsPanel.add(boolBox, "br hfill");
		featureOptionsPanel.add(removeStopBox, "br hfill");
		featureOptionsPanel.add(stemBox, "br hfill");

		parentPanel.add(featureTypesPanel);
		parentPanel.add(featureOptionsPanel);
	}

	@Override
	protected Component getConfigurationUIForSubclass() {
		return parentPanel;
	}

	public void setOption(String tag, boolean value){
		if(tag.equals(punctTag)){
			isPunct = value;
		}else if(tag.equals(uniTag)){
			isUni = value;
		}else if(tag.equals(biTag)){
			isBi = value;
		}else if(tag.equals(triTag)){
			isTri = value;
		}else if(tag.equals(posBiTag)){
			isPosBi = value;
		}else if(tag.equals(binaryTag)){
			optBool = value;
		}else if(tag.equals(lineLenTag)){
			isLineLen = value;
		}
	}
	
	public void setAllOptionsFalse(){
		isPunct = false;
		isUni = false;
		isBi = false;
		isTri = false;
		isPosBi = false;
		optBool = true;
		isLineLen = false;
		isContNonStop = false;
		optRemoveStop = false;
		optStem = false;
	}
	
	/**
	 * Tags for XML.
	 */
	public String punctTag = "punctuation";
	public String uniTag = "unigrams";
	public String biTag = "bigrams";
	public String triTag = "trigrams";
	public String posBiTag = "posBigrams";
	public String binaryTag = "binaryNgrams";
	public String lineLenTag = "lineLength";
	public String contNonStopTag = "containsNonstop";
	public String removeStopTag = "removeStop";
	public String stemTag = "stem";


	public void configureFromFile(String filename){
		o = new OptionsModel();
		isPunct = false;  isUni = false;  isBi = false;  isTri = false;
		isPosBi = false;  optBool = false;  isLineLen = false;
		isContNonStop = false;  optRemoveStop = false;  optStem = false;
		try{
			BufferedReader in = new BufferedReader(new FileReader(filename));
			String line;
			while((line = in.readLine()) != null){
				if(line.equals(punctTag)){
					isPunct = true;
				}else if(line.equals(uniTag)){
					isUni = true;
				}else if(line.equals(biTag)){
					isBi = true;
				}else if(line.equals(triTag)){
					isTri = true;
				}else if(line.equals(posBiTag)){
					isPosBi = true;
				}else if(line.equals(binaryTag)){
					optBool = true;
				}else if(line.equals(lineLenTag)){
					isLineLen = true;
				}else if(line.equals(contNonStopTag)){
					isContNonStop = true;
				}else if(line.equals(removeStopTag)){
					optRemoveStop = true;
				}else if(line.equals(stemTag)){
					optStem = true;
				}
			}
			in.close();
		}catch(Exception e){
			e.printStackTrace();
		}
		updateOptionsModel();
	}

	public String getOutputName(){ return "th"; }

	@Override
	public void uiToMemory(){
		o = new OptionsModel();
		updateOptionsModel();
	}
	
	public OptionsModel getOptionsModel(){
		return o;
	}

	private void updateOptionsModel() {
		o.setPunctuation(isPunct);
		o.setUnigrams(isUni);
		o.setBigrams(isBi);
		o.setTrigrams(isTri);
		o.setPosBigrams(isPosBi);
		o.setLineLength(isLineLen);
		o.setContainsNonStopword(isContNonStop);
		o.setRemoveStopwords(optRemoveStop);
		o.setStem(optStem);
		o.setStopwordsAll(true);
	}

	/**
	 * Loop through documents, aggregating features as we go, extracting according to the settings in the OptionsModel.
	 */
	@Override
	public Collection<FeatureHit> extractFeatureHitsForSubclass(
			DocumentListInterface documents, JLabel progress) {
		ArrayList<FeatureHit> results = new ArrayList<FeatureHit>();
		FeatureSelector fs = new FeatureSelector(o);
		List<String> corpus = documents.getCoveredTextList();
		double time1 = System.currentTimeMillis();
		Map<String, Feature> allFeatures = new TreeMap<String, Feature>();
		int index = 0;
		for(String document : corpus){
			if(halt){
				halt = false;
				return null;
			}
			progress.setText("Extracting from document " + index);
			String doc = HelperFunctions.replacePuncWithSpecialTokens(document.toString(), Tokenizer.LANGS[Tokenizer.LANG_ENGLISH]);
			THashMap feats = fs.getFeatureSet(doc);
			double tot = 0.0;
			if(!optBool){
				for(Object feat : feats.keySet()){
					tot += (Double)feats.get(feat);
				}
			}
			Map<String, Set<Integer>> hits = new TreeMap<String, Set<Integer>>();
			Map<String, Integer> lengths = new TreeMap<String, Integer>(); 
			Map<String, IndexedFeature> originals = new TreeMap<String, IndexedFeature>();
			for(Object o : feats.keySet()){
				IndexedFeature feat = (IndexedFeature)o;
				originals.put(feat.getFeature(), feat);
				if(!hits.containsKey(feat.getFeature())){
					hits.put(feat.getFeature(), new TreeSet<Integer>());
				}
				hits.get(feat.getFeature()).add(feat.getIndex());
			}
			for(String feat : hits.keySet()){
				Feature f;
				if(!allFeatures.containsKey(feat)){
					Feature.Type type = Type.BOOLEAN;
					if(!optBool){
						type = Type.NUMERIC;
					}
					if(feat.equals(FeatureSelector.LINE_LENGTH)){
						type = Type.NUMERIC;
					}
					f = Feature.fetchFeature(getOutputName(), feat.toString(), type);
					allFeatures.put(feat, f);	
				}else{
					f = allFeatures.get(feat);
				}

				Double value = 0.0+hits.get(feat).size();
				Integer length = originals.get(feat).getLength();
				if(hits.get(feat).size()>=0){
					boolean loc = false;
					for(Integer ind : hits.get(feat)){
						if(ind>=0) loc = true;
					}
					if(loc){
						Collection<int[]> toPass = new ArrayList<int[]>();
						for(Integer hit : hits.get(feat)){
							toPass.add(new int[]{hit,hit+length});
						}
						if(optBool){	
							results.add(new LocalFeatureHit(f, value > 0.0 ? Boolean.TRUE : Boolean.FALSE, index, toPass));
						}else{
							results.add(new LocalFeatureHit(f, value, index, toPass));				
						}
					}else{
						if(feat.equals(FeatureSelector.LINE_LENGTH) || !optBool){
							results.add(new FeatureHit(f, value, index));				
						}else{
							results.add(new FeatureHit(f, value > 0.0 ? Boolean.TRUE : Boolean.FALSE, index));
						}
						
					}
				}
			}
			index++;
		}
		double time2 = System.currentTimeMillis();		
		progress.setText("");
		return results;
	}

	public String toString(){
		return "TagHelperTools (Basic Text Features)";
	}

	@Override
	public void stopWhenPossible(){
		halt = true;
	}

}
