package plugin.simple.svml;
/*
 * Copyright (c) 2008, Lawrence Livermore National Security, LLC. Produced at the Lawrence Livermore National
 * Laboratory. Written by David Buttler, buttler1@llnl.gov CODE-400187 All rights reserved. This file is part of
 * RECONCILE
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
 * License (as published by the Free Software Foundation) version 2, dated June 1991. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the terms and conditions of the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA For full text see license.txt
 */


import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;


/**
 * @author <a href="mailto:buttler1@llnl.gov">David Buttler </a>
 * 
 */
public class InputStreamLineIterable
    implements Iterable<String> {


private InputStream mIn;
private boolean itCalled;

public InputStreamLineIterable(InputStream in)
{
  if (in == null) {
    throw new RuntimeException("input stream is null");
  }
  mIn = in;

}



public Iterator<String> iterator()
{
  if (itCalled) {
    throw new RuntimeException("Error, you can only iterate once over an input stream");
  }
  itCalled = true;
  try {
    return new InputStreamLineIterator(mIn);
  }
  catch (IOException e) {
    throw new RuntimeException(e);
  }
}

/**
 * Iterate over lines of an uncompressed or a gzip file There is some overhead in checking to see if the file is
 * compressed.
 * 
 * @param f
 * @return
 * @throws IOException
 */
public static InputStreamLineIterable iterate(InputStream f)
    throws IOException
{
  if (f == null) {
    throw new NullPointerException("input stream is null");
  }
  return new InputStreamLineIterable(f);
}
}
