package plugin.simple.svml;
/*
 * Copyright (c) 2008, Lawrence Livermore National Security, LLC. Produced at the Lawrence Livermore National

 * Laboratory. Written by David Buttler, buttler1@llnl.gov CODE-400187 All rights reserved. This file is part of
 * RECONCILE
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
 * License (as published by the Free Software Foundation) version 2, dated June 1991. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the terms and conditions of the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA For full text see license.txt
 */


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * @author <a href="mailto:buttler1@llnl.gov">David Buttler </a>
 * 
 */
public class InputStreamLineIterator
    implements Iterator<String> {

private LineIterator mLI;

private BufferedReader mBR;

private Reader mFR;

private boolean mCloseReader = true;

public InputStreamLineIterator(InputStream in)
    throws IOException {
  if (in == null) {
    throw new IOException("input stream is null");
  }
  mFR = new InputStreamReader(in);
  init();

}

public InputStreamLineIterator(Reader r)
    throws IOException {
  mFR = r;
  init();
  mCloseReader = false;
}

private void init()
    throws IOException
{
  mBR = new BufferedReader(mFR);
  mLI = new LineIterator(mBR);
}

/*
 * (non-Javadoc)
 *
 * @see java.util.Iterator#hasNext()
 */
public boolean hasNext()
{

  boolean next = false;
  if (mLI != null) {
    next = mLI.hasNext();
  }
  if (!next) {
    close();
  }
  return next;
}

/*
 * (non-Javadoc)
 *
 * @see java.util.Iterator#next()
 */
public String next()
{
  if (mLI == null) throw new NoSuchElementException("no lines to process");
  String next = null;
  try {
    next = mLI.next();
  }
  catch (NoSuchElementException e) {
    close();
    throw e;
  }

  return next;
}

private void close()
{
  try {
    if (mBR != null) mBR.close();
  }
  catch (Exception e) {
    e.printStackTrace();
    throw new RuntimeException(e);
  }
  if (mCloseReader) {
    try {
      if (mFR != null) mFR.close();
    }
    catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException(e);
    }
  }
}

/*
 * (non-Javadoc)
 *
 * @see java.util.Iterator#remove()
 */
public void remove()
{
  // ignore
}

}
