package plugin.simple.svml;
/*
 * Copyright (c) 2008, Lawrence Livermore National Security, LLC. Produced at the Lawrence Livermore National
 * Laboratory. Written by David Buttler, buttler1@llnl.gov CODE-400187 All rights reserved. This file is part of
 * RECONCILE
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
 * License (as published by the Free Software Foundation) version 2, dated June 1991. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the terms and conditions of the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA For full text see license.txt
 */


import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

/**
 * Iterates over the lines of a file, a reader, or a plain string. File must be uncompressed
 * 
 * @author <a href="mailto:buttler1@llnl.gov">David Buttler </a>
 * 
 */
public class LineIterable
    implements Iterable<String> {


private File mFile;
private boolean recallable;
private boolean itCalled;
private BufferedReader mReader;
private String string;

public LineIterable(File f)
    throws IOException {
  if (!f.exists()) {
    throw new IOException("File does not exist: " + f.getAbsolutePath());
  }
  mFile = f;
  recallable = true;
  itCalled = false;
}

public LineIterable(String contents) {
  string = contents;
  recallable = true;
}


public Iterator<String> iterator()
{
  if (recallable) {
    if (string != null) {
      try {
        return new LineIterator(string);
      }
      catch (IOException e) {
        throw new RuntimeException(e);
      }
    }
    try {
      return new LineIterator(mFile);
    }
    catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
  else {
    if (itCalled) {
      throw new RuntimeException("Error, you can only iterate once over an input stream");
    }
    itCalled = true;

  try {
      return new LineIterator(mReader);
  }
  catch (IOException e) {
    throw new RuntimeException(e);
  }
  }
}

/**
 * Iterate over lines of an uncompressed file
 * 
 * @param f
 * @return
 * @throws IOException
 */
public static LineIterable iterate(File f)
    throws IOException
{
  if (f == null) {
    throw new NullPointerException("input File is null");
  }
  if (!f.exists()) {
    throw new IOException("File does not exist: " + f.getAbsolutePath());
  }
  return new LineIterable(f);
}

/**
 * Iterate over lines of an uncompressed file compressed.
 * 
 * @param f
 * @return
 * @throws IOException
 */
public static LineIterable iterate(String f)
    throws IOException
{
  return iterate(new File(f));
}


}
