package plugin.simple.svml;
/*
 * Copyright (c) 2008, Lawrence Livermore National Security, LLC. Produced at the Lawrence Livermore National
 * Laboratory. Written by David Buttler, buttler1@llnl.gov CODE-400187 All rights reserved. This file is part of
 * RECONCILE
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
 * License (as published by the Free Software Foundation) version 2, dated June 1991. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the terms and conditions of the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA For full text see license.txt
 */



import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterate over lines
 * 
 * @author <a href="mailto:buttler1@llnl.gov">David Buttler </a>
 * 
 */
public class LineIterator
    implements Iterator<String> {


private BufferedReader mReader;

private String mLine;

/**
 * Iterate over a File line by line
 * 
 * @param br
 *          the buffered reader to iterate over
 * @throws IOException
 */
public LineIterator(File f)
    throws IOException {
  BufferedReader br = new BufferedReader(new FileReader(f));
  mReader = br;
  mLine = mReader.readLine();

}

/**
 * Iterate over a BufferedReader line by line
 * 
 * @param br
 *          the buffered reader to iterate over
 * @throws IOException
 */
public LineIterator(BufferedReader br)
    throws IOException {
  mReader = br;
  mLine = mReader.readLine();

}

/**
 * Iterate over a string line by line
 * 
 * @param string
 * @throws IOException
 * @throws IOException
 */
public LineIterator(String string)
    throws IOException {
  this(new BufferedReader(new StringReader(string)));
}

/*
 * (non-Javadoc)
 * 
 * @see java.util.Iterator#hasNext()
 */
public boolean hasNext()
{
  return mLine != null;
}

/*
 * (non-Javadoc)
 * 
 * @see java.util.Iterator#next()
 */
public String next()
{
  if (mLine != null) {
    String result = mLine;
    try {
      mLine = mReader.readLine();
      return result;
    }
    catch (IOException e) {
      System.out.println("error in iterating over file:" + e.getMessage());
    }
  }
  else {
    throw new NoSuchElementException("No more lines in the file");
  }
  return null;
}

/*
 * (non-Javadoc)
 * 
 * @see java.util.Iterator#remove()
 */
public void remove()
{
  // ignore
}

}
