package plugin.simple.svml;


import edu.cmu.side.SimpleWorkbench;
import edu.cmu.side.simple.feature.Feature;
import edu.cmu.side.simple.feature.FeatureHit;
import edu.cmu.side.simple.feature.FeatureTable;
import edu.cmu.side.simple.feature.Feature.Type;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.JLabel;

import weka.classifiers.evaluation.NominalPrediction;


public class SVMLToolkit {

	private static String svmlPath = SimpleWorkbench.toolkitsFolder + "/svmlight/";
	private static String learnCall = svmlPath+"svm_learn";
	private static String classifyCall = svmlPath+"svm_classify";

	private static boolean halt = false;

	public static Map<String, String> train(FeatureTable ft, Map<Integer, Integer> foldsMap, JLabel progressLabel) throws Exception{
		StringBuilder[] docs = produceSVMLStrings(ft);
		try{
			BufferedWriter out = new BufferedWriter(new FileWriter(svmlPath+ft.getTableName().replace(" ", "")+".train"));
			StringBuilder sb = new StringBuilder();
			for(StringBuilder d : docs){
				sb.append(d.toString() + "\n");
			}
			out.write(sb.toString());
			out.close();
			BufferedWriter testOut = null;
			ArrayList<Integer> folds = new ArrayList<Integer>();
			for(int i : foldsMap.keySet()) if(!folds.contains(foldsMap.get(i))) folds.add(foldsMap.get(i));

			Map<Integer, ArrayList<Integer>> indexMap = new TreeMap<Integer, ArrayList<Integer>>();

			for(int i : folds){
				if(halt){
					halt = false;
					return null;
				}
				progressLabel.setText("Writing fold " + i);
				out = new BufferedWriter(new FileWriter(svmlPath+ft.getTableName().replace(" ", "")+".fold"+i+".train"));
				StringBuilder train = new StringBuilder();
				StringBuilder test = new StringBuilder();
				for(int j = 0; j <  docs.length; j++){
					if(foldsMap.get(j)==i){
						test.append(docs[j] + "\n");						
					}else{
						train.append(docs[j] + "\n");
					}
				}
				ArrayList<Integer> index = new ArrayList<Integer>();
				Map<Integer, Integer> foldMap = ft.foldIndexToIndex(foldsMap, i);
				for(int j = 0; j < foldMap.keySet().size(); j++){
					index.add(foldMap.get(j));
				}
				indexMap.put(i, index);
				out.write(train.toString());
				out.close();
				testOut = new BufferedWriter(new FileWriter(svmlPath+ft.getTableName().replace(" ", "")+".fold"+i+".test"));
				testOut.write(test.toString());
				testOut.close();
			}
			return evaluate(ft, folds, indexMap, progressLabel);
		}catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}

	private static StringBuilder[] produceSVMLStrings(FeatureTable ft)
	throws Exception {
		Set<Feature> ftks = ft.getFeatureSet();
		ArrayList<String> annots = ft.getDocumentList().getAnnotationArray();
		StringBuilder[] docs = new StringBuilder[annots.size()];
		List<String> labels = new ArrayList<String>();
		for(String s : ft.getDocumentList().getLabelArray()) labels.add(s);
		if(ft.getClassValueType() != Type.NUMERIC && labels.size() != 2){
			throw new Exception("SVMLight only handles binary classification at this time.");
		}
		for(int i = 0; i < docs.length; i++){
			switch(ft.getClassValueType()){
			case STRING:
			case NOMINAL:
			case BOOLEAN:				
				if(labels.get(0).equals(annots.get(i))){
					docs[i] = new StringBuilder("-1 ");
				}else if(labels.get(1).equals(annots.get(i))){
					docs[i] = new StringBuilder("1 ");
				}else{
					docs[i] = new StringBuilder("ERROR ");
				}
				break;
			case NUMERIC:
				docs[i] = new StringBuilder(annots.get(i)+" ");
				break;
			}
		}
		Map<String, Integer> featureNums = new HashMap<String, Integer>(ftks.size());
		int featureIndex = 1;
		double timePutting = 0.0;
		double timeGetting = 0.0;
		double timeAppending = 0.0;
		for(Feature f : ftks){
			double time1 = System.currentTimeMillis();
			String featName = f.getFeatureName();
			switch(f.getFeatureType()){
			case STRING:
			case NOMINAL:
				for(String value : f.getNominalValues()){
					featureNums.put(featName+":"+value, featureIndex++);
				}
				break;
			case NUMERIC:
			case BOOLEAN:
				featureNums.put(featName, featureIndex++);
				break;
			}
			double time2 = System.currentTimeMillis();
			Collection<FeatureHit> matches = ft.getHitsForFeature(f);
			double time3 = System.currentTimeMillis();
			for(FeatureHit hit : matches){
				int m = hit.getDocumentIndex();
				switch(f.getFeatureType()){
				case NUMERIC:
					docs[m].append(featureNums.get(featName)+":"+hit.getValue()+" ");
					break;
				case STRING:
				case NOMINAL:
					docs[m].append(featureNums.get(featName+":"+hit.getValue().toString()));
				case BOOLEAN:
					if(hit.getValue().equals(Boolean.TRUE)){
						docs[m].append(featureNums.get(featName));
					}
					docs[m].append(":1 ");
					break;
				}
			}
			double time4 = System.currentTimeMillis();
			timePutting += (time2-time1);
			timeGetting += (time3-time2);
			timeAppending += (time4-time3);
		}
		BufferedWriter out = new BufferedWriter(new FileWriter(svmlPath+"featureNames"));
		BufferedWriter alone = new BufferedWriter(new FileWriter(svmlPath+"alone"));
		alone.write("-1 \n");
		for(String s : featureNums.keySet()){
			out.write(s+"\n");
			alone.write("-1 " + featureNums.get(s) + ":1 \n");
		}
		out.close();
		alone.close();
		return docs;
	}

	public static SortedMap<String, String> evaluate(FeatureTable ft, ArrayList<Integer> folds, Map<Integer, ArrayList<Integer>> indexMap, JLabel progressLabel){
		String name = ft.getTableName().replace(" ", "");
		SortedMap<String, String> out = new TreeMap<String, String>();
		Map<Integer, String> predicted = new TreeMap<Integer, String>();
		ArrayList<String> filenames = new ArrayList<String>();
		try{
			ArrayList<String> train = new ArrayList<String>();
			for(int i : folds){
				if(halt){
					halt = false;
					return null;
				}
				progressLabel.setText("Training fold " + i);
				train.addAll(runExternalCaptureOutput(learnCall + (ft.getClassValueType().equals(Type.NUMERIC)?" -z r ":" ") + svmlPath + name + ".fold"+i+".train " + svmlPath + name + ".fold"+i+".model"));

			}
			ArrayList<String> test = new ArrayList<String>();
			for(int i : folds){
				if(halt){
					halt = false;
					return null;
				}
				progressLabel.setText("Testing fold " + i);
				test.addAll(runExternalCaptureOutput(classifyCall + " " + svmlPath + name + ".fold"+i+".test " + svmlPath + name + ".fold"+i+".model " + svmlPath + name + ".fold"+i+".pred"));	
				filenames.add(svmlPath + name + ".fold"+i);
			}
			for(int i : folds){
				if(halt){
					halt = false;
					return null;
				}
				progressLabel.setText("Getting predictions from fold " + i);
				BufferedReader in = new BufferedReader(new FileReader(svmlPath+name+".fold"+i+".pred"));
				String pred;
				int j = 0;
				while((pred = in.readLine()) != null){
					double p = Double.parseDouble(pred);
					boolean above = p>0;
					predicted.put(indexMap.get(i).get(j++), ft.getDocumentList().getLabelArray()[above?1:0]);
				}
				in.close();
			}
			if(halt){
				halt = false;
				return null;
			}

			progressLabel.setText("Training final model.");
			StringBuilder trOut = new StringBuilder();
			for(String t : train) trOut.append(t+"\n");
			StringBuilder teOut = new StringBuilder();
			for(String t : test) teOut.append(t+"\n");
			List<String> fin = runExternalCaptureOutput(learnCall + " " + svmlPath + name + ".train " + svmlPath + name + ".model");
			StringBuilder finOut = new StringBuilder();
			for(String f : fin) finOut.append(f+"\n");
			if(halt){
				halt = false;
				return null;
			}

			runExternalCaptureOutput(classifyCall + " " + svmlPath + "alone " + svmlPath + name + ".model " + svmlPath + "alone.pred");
			Map<Feature, Comparable> weights = new HashMap<Feature, Comparable>();
			StringBuilder weightBuilder = new StringBuilder();
			BufferedReader names = new BufferedReader(new FileReader(svmlPath+"featureNames"));
			BufferedReader wts = new BufferedReader(new FileReader(svmlPath+"alone.pred"));
			Double zero = Double.parseDouble(wts.readLine());
			String weight;
			while((weight = wts.readLine()) != null){
				String feature = names.readLine();
				Double wt = Double.parseDouble(weight)-zero;
				weightBuilder.append(feature + ", " + wt + "\n");
				for(Feature f : ft.getFeatureSet()){
					if(f.getFeatureName().equals(feature)){
						weights.put(f, wt);
					}
				}
			}
			names.close();
			wts.close();
			int w = 1;
			while(ft.getEvaluations().containsKey("weights"+w)) w++;
			ft.addEvaluation("weights"+w, weights);
			String summary = summarize(ft, predicted);

			StringBuilder predictions = new StringBuilder();
			for(int i = 0; i < predicted.keySet().size(); i++){
				predictions.append(predicted.get(i) + "\n");
			}
			out.put("summary", summary);
			out.put("cv-fold-predictions", predictions.toString());
			out.put("cv-train", trOut.toString());
			out.put("cv-test", teOut.toString());
			out.put("weights", weightBuilder.toString());
			out.put("train", finOut.toString());
		}catch(Exception e){
			e.printStackTrace();
		}
		return out;
	}

	public static String summarize(ArrayList<String> test){
		ArrayList<Double> folds = new ArrayList<Double>();
		for(String s : test){
			if(s.contains("Accuracy on test set")){
				folds.add(Double.parseDouble(s.split(" ")[4].replace("%", "")));
			}
		}
		Double tot = 0.0; for(Double d : folds) tot += d; tot /= folds.size();
		String result = "CV Accuracy: " + tot;
		return result;
	}

	public static String summarize(FeatureTable f, Map<Integer, String> predicted){
		String result = "CV Accuracy: ";
		System.out.println(predicted.values().size() + " values");
		double counter = 0.0;
		ArrayList<String> act = f.getDocumentList().getAnnotationArray();
		switch(f.getClassValueType()){
		case NOMINAL:
		case BOOLEAN:
		case STRING:
			ArrayList<Double> folds = new ArrayList<Double>();
			StringBuilder foldsString = new StringBuilder();
			for(int i = 0; i < act.size(); i++){
				if(act.get(i).equals(predicted.get(i))){
					counter++;
				}
			}
			result += (counter/act.size()) + "\n";
			break;
		case NUMERIC:
			double predict = 0.0;
			double actual = 0.0;
			for(int i = 0; i < act.size(); i++){
				predict += Double.parseDouble(predicted.get(i));
				actual += Double.parseDouble(act.get(i));
			}
			predict /= act.size();
			actual /= act.size();
			double num = 0.0;
			double pDenom = 0.0;
			double aDenom = 0.0;
			for(int i = 0; i < act.size(); i++){
				double p = Double.parseDouble(predicted.get(i));
				double a = Double.parseDouble(act.get(i));
				double pFail = p - predict;
				double aFail = a - actual;
				num += pFail*aFail;
				pDenom += Math.pow(pFail, 2);
				aDenom += Math.pow(aFail, 2);
			}
			double den = Math.sqrt(pDenom)*Math.sqrt(aDenom);
			result += (num/den) + "\n";
			break;
		}
		return result;

	}

	public static ArrayList<String> runExternalCaptureOutput(String command)
	throws IOException, InterruptedException
	{
		Process p = Runtime.getRuntime().exec(command);
		ArrayList<String> lines = new ArrayList<String>();

		for (String line : InputStreamLineIterable.iterate(p.getInputStream())) {
			lines.add(line);
		}

		p.waitFor();
		if (p.exitValue() != 0) {
			for (String line : InputStreamLineIterable.iterate(p.getErrorStream())) {
				System.err.println(line);
			}
			throw new RuntimeException("Couldn't run " + command);
		}

		return lines;
	}

	public static void halt(){
		halt = true;
	}
}
